package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import java.awt.geom.Point2D;

import xmdv.dataproc.Brush;
import xmdv.dataproc.Brush1D;
import xmdv.dataproc.Dataset;



/**

 * This class is an implementation of the abstract class FlatDisplay using 

 * the Dimensional Stacking technique.

 *

 * @author  Jing Yang

 * @version v1.0

 */

public class FlatDimstack extends FlatDisplay{

    private Point2D.Double prevPosition;

    private int xDimNum;

    private int yDimNum;

    private int[] xDimIndexes;

    private int[] yDimIndexes;

    private int[] xDimCards;

    private int[] yDimCards;

    private int[] xDimBinNums;

    private int[] yDimBinNums;

    private int xBinNum;

    private int yBinNum;

    private int binNum;

    boolean[] binFilled;

    private double xBinSize;

    private double yBinSize;

    

    private int[] xBinIndexes;

    private int[] yBinIndexes;

    

    private int quickDrawXBinIndex;

    private int quickDrawYBinIndex;

    

    /* inits: pt = point, pts = points, lo = low, hi = high, num = number 

     *        nor = normalized, prev = previous, plotIndex = dimension 

     *        min = minimum, max = maximum */

        

    /** Creates a new instance of FlatDimstack display */

    public FlatDimstack () {

    }

    

    public FlatDimstack(grCanvas canvas, ColorManager colorManager, Dataset dataset) {

        setCanvas(canvas);

        setColorManager(colorManager);

        setDataset(dataset);

    }

    

    protected boolean isDimensionalityValid() {

        if ( dataset.getCurrDimNum() < 1 ) {

            System.out.println("No sufficent dimensions for this kind of display.");

            return false;

        } else 

            return true;

    }

    

    protected void setParameters() {

        super.setParameters();

        

        int i;

        

        if (dimNum != 0) {

            yDimNum = dimNum / 2;

            xDimNum = dimNum - yDimNum;

            

            if (xDimNum > 4)

                xDimNum = 4;

            if (yDimNum > 4)

                yDimNum = 4;

                

            xDimIndexes = new int[xDimNum];

            yDimIndexes = new int[yDimNum];

            xDimCards = new int[xDimNum];

            yDimCards = new int[yDimNum];

            xDimBinNums = new int[xDimNum];

            yDimBinNums = new int[yDimNum];

            

            xBinIndexes = new int[xDimNum];

            yBinIndexes = new int[yDimNum];

            

            xBinNum = 1;

            for (i = 0; i < xDimNum; i++) {

                xDimIndexes[i] = i * 2;

                xDimCards[i] = dims[xDimIndexes[i]].getCardinality();

                xBinNum *= xDimCards[i];

            }

            yBinNum = 1;

            for (i = 0; i < yDimNum; i++) {

                yDimIndexes[i] = i * 2 + 1;

                yDimCards[i] = dims[yDimIndexes[i]].getCardinality();

                yBinNum *= yDimCards[i];

            }

            

            xDimBinNums[xDimNum-1] = 1;

            for (i = xDimNum-2; i >= 0; i--) {

                xDimBinNums[i] = xDimBinNums[i+1] * xDimCards[i+1];

            }

            yDimBinNums[yDimNum-1] = 1;

            for (i = yDimNum-2; i >= 0; i--) {

                yDimBinNums[i] = yDimBinNums[i+1] * yDimCards[i+1];

            }

            

            xBinSize = 1./xBinNum;

            yBinSize = 1./yBinNum;

            

            binNum = xBinNum * yBinNum;

            binFilled = new boolean[binNum];

            for (i = 0; i < binNum; i++)

                binFilled[i] = false;

        }

    }

    

    

    protected void drawSingleData(double[] norData, int dataIndex) {

        int xBinIndex = getXBinIndex(norData);

        int yBinIndex = getYBinIndex(norData);

        int binIndex = yBinIndex * xBinNum + xBinIndex;

        

        if (binFilled[binIndex])

            return;

        else {

            binFilled[binIndex] = true;

            drawSingleBin(xBinIndex, yBinIndex);

            return;

        }

    }

    

    protected void drawRegularDataCleanUp() {

        for (int i = 0; i < binNum; i++)

            binFilled[i] = false;

    }

    

    protected void drawHighlightedDataCleanUp() {

        for (int i = 0; i < binNum; i++)

            binFilled[i] = false;

    }

    

    

    protected void drawSingleBin(int xBinIndex, int yBinIndex) {

        Point2D.Double pt1 = new Point2D.Double(xBinIndex * xBinSize, yBinIndex * yBinSize);

        Point2D.Double pt2 = new Point2D.Double((xBinIndex+1) * xBinSize, (yBinIndex+1) * yBinSize);

        canvas.fillRectangle(pt1, pt2);

    }

    

    protected void drawSingleBrush(Brush brush) {

        int i, j, temp, xBinIndex, yBinIndex;

        boolean isOutside;

        double min, max;

    

        canvas.setForegroundColor(colorManager.getColor("brush"));

    

        for (i = 0; i < binNum; i++) {

            isOutside = false;

            xBinIndex = i % xBinNum;

            yBinIndex = i / xBinNum;

            temp = xBinIndex;

            for (j = 0; j < xDimNum; j++) {

                xBinIndexes[j] = temp / xDimBinNums[j];

                temp = temp % xDimBinNums[j];

                min = xBinIndexes[j] * 1. / xDimCards[j];

                max = min + 1./xDimCards[j];

                if (!isWithin(min, max, brush.getBrush1D(xDimIndexes[j]))) {

                    isOutside = true;

                    break;

                }

            }

            if (!isOutside) {

                temp = yBinIndex;

                for (j = 0; j < yDimNum; j++) {

                    yBinIndexes[j] = temp / yDimBinNums[j];

                    temp = temp % yDimBinNums[j];

                    

                    min = yBinIndexes[j] * 1. / yDimCards[j];

                    max = min + 1./yDimCards[j];

                    if (!isWithin(min, max, brush.getBrush1D(yDimIndexes[j]))) {

                        isOutside = true;

                        break;

                    }

                }

            }

            if (!isOutside) {

                drawSingleBin(xBinIndex, yBinIndex);

            }

        }

    }

    

    protected void drawAxes()

    {

        int i, j;

        double temp;

        Point2D.Double pt1 = new Point2D.Double();

        Point2D.Double pt2 = new Point2D.Double();

        canvas.setLineWidth(1.);

    

        for (i = xDimNum - 1; i >= 0; i--) {

            canvas.setForegroundColor(colorManager.getColor("grid"+i));

            for (j = 0; j <= xBinNum; j += xDimBinNums[i]) {

                temp = j * xBinSize;

                pt1.setLocation(temp, 0);

                pt2.setLocation(temp, 1);

                canvas.drawLine(pt1, pt2);

            } 

        }

        for (i = yDimNum - 1; i >= 0; i--) {

            canvas.setForegroundColor(colorManager.getColor("grid"+i));

            for (j = 0; j <= yBinNum; j += yDimBinNums[i]) {

                temp = j * yBinSize;

                pt1.setLocation(0, temp);

                pt2.setLocation(1, temp);

                canvas.drawLine(pt1, pt2);

            } 

        }

    }

    

    protected void inputMouseMotion(Point2D.Double pt)

    {

        if (inputMode == CANVAS_PAN) {

            canvas.pan(prevPosition, pt);

            canvas.refresh();

        }

        else {

            int[] xBinIndex = new int[1];

            if (!getXBinIndex(pt, xBinIndex)) {

                return;

            }

            int[] yBinIndex = new int[1];

            if (!getYBinIndex(pt, yBinIndex)) {

                return;

            }

            

            dimstackCaption(xBinIndex[0], yBinIndex[0]);

        

            switch (inputMode) {

                case TRANSLATE_BRUSH:

                    inputDimstackMoveBrush(xBinIndex[0], yBinIndex[0]);

                    dataset.refreshAllCanvas();

                    break;

        

                case MULTI_SELECT:

                    inputDimstackMultiSelect(xBinIndex[0], yBinIndex[0]);

                    dataset.refreshAllCanvas();

                    return;

        

                default: break;

            }

            prevPosition = pt;

        }    

    }

    

    

    protected void inputButtonPress(int button, Point2D.Double pt) {

        inputMode = NONE;

        prevPosition = pt;

    

        if ((!canvas.altkey) && (!canvas.controlkey) && button == 3) {

            inputMode = CANVAS_PAN;

            canvas.setMoveCursor();

            return;

        }  

    

        if (canvas.shiftkey && button == 1) {

            inputMode = MULTI_SELECT;

        } else if (button == 1) {

            inputMode = TRANSLATE_BRUSH;

        }

    

        inputMouseMotion(pt);

    }

    

    

    protected void inputButtonRelease(int button, Point2D.Double pt)

    {

        switch (inputMode) {

    

            case CANVAS_PAN:

                canvas.setDefaultCursor();

                break;

    

            case MULTI_SELECT:

                break;

    

            case TRANSLATE_BRUSH:

                dataset.brush();

                canvas.setDefaultCursor();

                dataset.refreshAllCanvas();

                break;

    

            default: break;

        }

    

        inputMode = NONE;

    }

    

    protected int getXBinIndex(double[] norData)

    {

        int i, index = 0;

        for (i = 0; i < xDimNum; i++) {

            xBinIndexes[i] = xDimBinNums[i] * (int)(norData[xDimIndexes[i]] * xDimCards[i]);        

            index += xBinIndexes[i];

        }

        return index;

    }

    

    protected int getYBinIndex(double[] norData)

    {

        int i, index = 0;

        for (i = 0; i < yDimNum; i++) {

            yBinIndexes[i] = yDimBinNums[i] * (int)(norData[yDimIndexes[i]] * yDimCards[i]);        

            index += yBinIndexes[i];

        }

        return index;

    }

    

    protected boolean getXBinIndex(Point2D.Double pt, int[] xBinIndex)

    {

        int index = (int)(pt.getX() / xBinSize);

        if (index < xBinNum) {

            xBinIndex[0] = index;

            return true;

        }

        else {

            return false;

        }

    }

    

    protected boolean getYBinIndex(Point2D.Double pt, int[] yBinIndex)

    {

        int index = (int)(pt.getY() / yBinSize);

        if (index < yBinNum) {

            yBinIndex[0] = index;

            return true;

        }

        else {

            return false;

        }

    }

    

    protected boolean isWithin(double min, double max, Brush1D brushDim) {  

        double brushDimMin = brushDim.getMin();

        double brushDimMax = brushDim.getMax();

        if (brushDimMin < min && brushDimMax < min)

            return false;

        else if (brushDimMin > max && brushDimMax > max)

            return false;

        else

            return true;    

    }

    

    protected boolean getCenterNorValue(int xBinIndex, int yBinIndex, double[] norValue) {

        int j, temp;

        double min, max;

    

        if (xBinIndex >= xBinNum)

            return false;

        if (yBinIndex >= yBinNum)

            return false;

            

        temp = xBinIndex;

        for (j = 0; j < xDimNum; j++) {

            xBinIndexes[j] = temp / xDimBinNums[j];

            temp = temp % xDimBinNums[j];

            min = xBinIndexes[j] * 1. / xDimCards[j];

            max = min + 1./xDimCards[j];

            norValue[xDimIndexes[j]] = (min + max) / 2;

        }

        temp = yBinIndex;

        for (j = 0; j < yDimNum; j++) {

            yBinIndexes[j] = temp / yDimBinNums[j];

            temp = temp % yDimBinNums[j];

            

            min = yBinIndexes[j] * 1. / yDimCards[j];

            max = min + 1./yDimCards[j];

            norValue[yDimIndexes[j]] = (min + max) / 2;

        }

        

        return true;

    }

    

    protected void inputDimstackMoveBrush(int xBinIndex, int yBinIndex) {

        double[] centerNorValue = new double[dimNum];

        getCenterNorValue(xBinIndex, yBinIndex, centerNorValue);

        

        for (int i = 0; i < dimNum; i++) {

            brush.shift(i, centerNorValue[i] - brush.getNorCenter(i));

        }

    }

    

    protected void inputDimstackMultiSelect(int xBinIndex, int yBinIndex)

    {

        int iXBinIndex, iYBinIndex;

        for (int i = 0; i < dataNum; i++)

        {

            double[] norData = dataset.getCurrNorData(i);

    

            iXBinIndex = getXBinIndex(norData);

            iYBinIndex = getYBinIndex(norData);

    

            if (iXBinIndex == xBinIndex && iYBinIndex == yBinIndex ) {

                if (dataset.multiSelectContains(i)) {

                    dataset.multiSelectRemovePoint(i);

    

                    quickDrawXBinIndex = xBinIndex;

                    quickDrawYBinIndex = yBinIndex;

                    quickDrawColor = colorManager.getColor("data");

                } else {

                    dataset.multiSelectAddPoint(i);

    

                    quickDrawXBinIndex = xBinIndex;

                    quickDrawYBinIndex = yBinIndex;

                    quickDrawColor = colorManager.getColor("paint");

                }

            }

        }

    }

    

    public void quickDraw() {

        canvas.setForegroundColor(quickDrawColor);

        drawSingleBin(quickDrawXBinIndex, quickDrawYBinIndex);

    }

    

    protected void dimstackCaption(int xBinIndex, int yBinIndex)

    {

        double[] centerNorValue = new double[dimNum];

        getCenterNorValue(xBinIndex, yBinIndex, centerNorValue);

    

        String valueMessage = "Bin Center:(";

        for (int i = 0; i < dimNum; i++) {

            valueMessage += dims[i].getNamedValueMessage(centerNorValue[i]) + " ";

        }

        valueMessage += ")";

        canvas.showMessage(valueMessage);

    }

}

