package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;



/**

 * This class defines the GLFont class to print strings in gl canvas

 *

 * @author  Jing Yang

 * @version v1.0

 */

public class GLFont implements gl4java.GLEnum {

    private int horiFontOffset;

    private int vertFontOffset;

    private int fontWidth;

    private int fontHeight;



    public GLFont(gl4java.GLFunc gl, Font font) {



        FontBits fontBits = getFontData(font);

        int width = fontBits.width;

        int height = fontBits.height;

        byte[][] bitmaps = fontBits.bits;



        fontWidth = width;

        fontHeight = height;



        gl.glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

        horiFontOffset = gl.glGenLists(128);

        for (int c=0; c < 128; c++) {

            gl.glNewList(horiFontOffset+c, GL_COMPILE);

            gl.glBitmap(width, height, 0.0f, 0.0f, (float)(width), 0.0f, bitmaps[c]);

            gl.glEndList();

        }



        vertFontOffset = gl.glGenLists(128);

        for (int c=0; c<128; c++) {

            gl.glNewList(vertFontOffset+c, GL_COMPILE);

            gl.glBitmap(width, height, 0.0f, 0.0f, 0.0f, (float)(height), bitmaps[c]);

            gl.glEndList();

        }

    }



    public int getFontWidth(char ch) {

        return fontWidth;

    }



    public int getFontHeight(char ch) {

        return fontHeight;

    }



    private FontBits getFontData(Font font) {

        BufferedImage image = new BufferedImage(256, 256, BufferedImage.TYPE_3BYTE_BGR);

        Graphics2D g2 = image.createGraphics();

        g2.setFont(font);

        FontMetrics metrics = g2.getFontMetrics(font);



        int width = 0;

        for (int c=0; c<128; c++) {

            if (width < metrics.charWidth(c))

                width = metrics.charWidth(c);

        }

        int height = metrics.getAscent() + metrics.getDescent();

        int x = 0;

        int y = metrics.getAscent();

        int bytesPerLine = (width + 7) / 8;

        byte[][] bitmaps = new byte[128][bytesPerLine*height];

        byte[] b = new byte[1];

        int[] rgb = new int [3];

        for (int c=0; c<128; c++) {

            b[0] = (byte) c;

            g2.setColor(java.awt.Color.white);

            g2.fillRect(0, 0, width, height);

            g2.setColor(java.awt.Color.black);

            g2.drawBytes(b, 0, 1, x, y);

            Raster raster = image.getData(new java.awt.Rectangle(0, 0, width, height));

            for (int i=0; i<height; i++) {

                int startIndex = bytesPerLine * i;

                for (int j=0; j<width; j++) {

                    if ((j % 8) == 0)

                        bitmaps[c][startIndex + (j/8)] = 0;

                    raster.getPixel(j, i, rgb);

                    if (rgb[0] == 0) {

                        bitmaps[c][startIndex + (j/8)] |= (byte) (1 << (7 - (j % 8)));

                    }

                }

            }

        }

        

        return new FontBits(width, height, bitmaps);

    }



    public void printHoriString(gl4java.GLFunc gl, double x, double y, String s) {

        byte[] b = s.getBytes();

        gl.glRasterPos2d(x, y);

        gl.glPushAttrib(GL_LIST_BIT);

        gl.glListBase(horiFontOffset);

        gl.glCallLists(b.length, GL_UNSIGNED_BYTE, b);

        gl.glPopAttrib();

    }

    

    public void printVertString(gl4java.GLFunc gl, double x, double y, String s) {

        byte[] b = s.getBytes();

        gl.glRasterPos2d(x, y);

        gl.glPushAttrib(GL_LIST_BIT);

        gl.glListBase(vertFontOffset);

        gl.glCallLists(b.length, GL_UNSIGNED_BYTE, b);

        gl.glPopAttrib();

    }

    

    private class FontBits {

        public int width;

        public int height;

        byte[][] bits;



        public FontBits(int w, int h, byte[][] bs) {

            this.width = w;

            this.height = h;

            this.bits = bs;

        }

    }

}

