package xmdv.graphics;



/***********************************************************

**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003

**  All Rights Reserved.

**

**  Author: Jing Yang

**  Date:   06/2003

***********************************************************/



import gl4java.swing.GLJPanel;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.*;
import java.awt.geom.Point2D;

import javax.swing.JScrollBar;
import javax.swing.JTextArea;



/**

 * This class provides the basic drawing operations. 

 * They are finally implemented using OpenGL through GL4Java.

 *

 * @author  Jing Yang

 * @version v1.0

 */

public class grCanvas extends GLJPanel 

                      implements KeyListener, MouseMotionListener, AdjustmentListener {

    // Variables declaration

    public static final double BORDER = 0.04;



    protected Point2D.Double zoom;

    protected Point2D.Double translate;

    

    protected boolean shiftkey = false;

    protected boolean controlkey = false;

    protected boolean altkey = false;

    

    protected GeneralDisplay display = null;;

    

    protected JScrollBar horiScrollBar = null;

    protected JScrollBar vertScrollBar = null;

    protected int oldHoriScrollValue = 0;

    protected int oldVertScrollValue = 0;

    protected int scrollStepNum = 1000;

    

    protected JTextArea textArea = null;

    

    protected GLFont font;

    

    protected boolean quickDraw = false;

    // End of variables declaration

    

    /** Creates new grCanvas */

    public grCanvas() {

        super(true);

        zoom = new Point2D.Double();

        translate = new Point2D.Double();

        display = null;

    

        addKeyListener(this);

        addMouseListener(this);

        addMouseMotionListener(this);

    }

    

    /** 

     * Sets horizental JScrollBar for this canvas 

     * @param scrollBar JScrollBar to be set

     */

    public void setHoriScrollBar(JScrollBar scrollBar) {

        this.horiScrollBar = scrollBar;

        positionHoriScrollBar();

    }

    

    /** 

     * Sets vertical JScrollBar for this canvas 

     * @param scrollBar JScrollBar to be set

     */

    public void setVertScrollBar(JScrollBar scrollBar) {

        this.vertScrollBar = scrollBar;

        positionVertScrollBar();

    }

    

    /** 

     * Sets JTextArea for this canvas 

     * @param textArea JTextArea to be set

     */

    public void setTextArea(JTextArea textArea) {

        this.textArea = textArea;

    }

    

    /**

     * shows a message in the text area of the canvas 

     * @param text the message to be shown

     */

    public void showMessage(String text) {

        if (textArea != null) {

            textArea.selectAll();

            textArea.replaceSelection(text);

            textArea.repaint();

        }

    }

    

    /** handles hortizental or vertical JScrollBar changes 

     * @param e AdjustmentEvent 

     */

    public void adjustmentValueChanged(AdjustmentEvent e) {

        if ( e.getAdjustable() == horiScrollBar )

           horiScrollBarChanged();

        else if (e.getAdjustable() == vertScrollBar)      

           vertScrollBarChanged();

    }

    

    /** 

     * sets display of this canvas 

     * @param display display to be drawn on this canvas

     */

    public void setDisplay(GeneralDisplay display) {

        this.display = display;

    }

    

    /** 

     * gets display of this canvas

     * @return display that is linked to this canvas

     */ 

    public GeneralDisplay getDisplay() {

        return display;

    }

    

    /** sets up transform matrix for this canvas */

    protected void setupTransformMatrix() {

        gl.glMatrixMode(GL_MODELVIEW);

        gl.glLoadIdentity();

        gl.glScaled(zoom.getX(), zoom.getY(), 1.0);

        gl.glTranslated(translate.getX(), translate.getY(), 1.0);

    }

    

    /** is called before init canvas */

    public void preInit() 

    {

    }

    

    

    /** init canvas */

    public void init() 

    {

        setZoom(1.0,1.0);

        setTranslate(0, 0);

    

        gl.glShadeModel(GL_FLAT);

        gl.glDisable(GL_DEPTH_TEST);

        font = new GLFont(gl, new java.awt.Font("Courier New", Font.PLAIN, 12));

    }

    

    /** reshapes the canvas */

    public void reshape(int width, int height) {

        gl.glViewport( 0, 0, width, height );

        gl.glMatrixMode(GL_PROJECTION);

        gl.glLoadIdentity();

        gl.glOrtho(-BORDER,1+BORDER,1+BORDER,-BORDER,-1,1);

        gl.glMatrixMode(GL_MODELVIEW);

    }

        

    /** draws the canvas */

    public void display() {

        setupTransformMatrix();

        if (display != null) {

            if (!quickDraw) {

                display.canvasDraw();

            }

            else {

                display.quickDraw();

            }

        }

        gl.glFlush();

    }

    

    

    /** 

     * checks if there is any input modifier within an InputEvent 

     * @param e the InputEvent to be checked 

     * @return true if control, alt, or shift key is down. Otherwise returns false 

     */  

    public boolean checkInputModifiers(InputEvent e) {

        boolean is_modifiers = false;

        controlkey = false;

        altkey = false;

        shiftkey = false;

        

        

        if (e.isControlDown()) {

            controlkey = true;

            is_modifiers = true;

        }

         

        if (e.isAltDown()) {

            altkey = true;

            is_modifiers = true;

        }

         

        if (e.isShiftDown()) {

            shiftkey = true;

            is_modifiers = true;

        }

        return is_modifiers;

    }

       

    /** mouse clicked handler */

    public void mouseClicked(MouseEvent e) {

    

    }

    

    /** mouse entered handler */

    public void mouseEntered(MouseEvent e) {

    

    }

    

    /** mouse exited handler */

    public void mouseExited(MouseEvent e) {

    

    }

    

    /** mouse pressed handler */

    public void mousePressed(MouseEvent e) {

        if (getGLContext() == null)

            return;

            

        getGLContext().gljMakeCurrent();

        

        if (display != null) 

            display.canvasInputButtonPress(e.getButton(), mapScreenToData(new Point2D.Double(e.getX(), e.getY())));

    

        getGLContext().gljFree();

    }

    

    /** mouse released handler */

    public void mouseReleased(MouseEvent e) {

        if (getGLContext() == null)

            return;

    

        getGLContext().gljMakeCurrent();

        

        if (display != null) 

                display.canvasInputButtonRelease(e.getButton(), mapScreenToData(new Point2D.Double(e.getX(), e.getY())) );

    

        getGLContext().gljFree();

    }

    

    /** mouse moved handler */

    public void mouseMoved(MouseEvent e) {

        if (getGLContext() == null)

            return;

    

        getGLContext().gljMakeCurrent();

        

        if (display != null) 

            display.canvasInputMouseMotion(mapScreenToData(new Point2D.Double(e.getX(), e.getY())) );

    

        getGLContext().gljFree();

    }

    

    /** mouse dragged handler */

    public void mouseDragged(MouseEvent e) {

        if (getGLContext() == null)

            return;

    

        getGLContext().gljMakeCurrent();

        

        if (display != null) 

            display.canvasInputMouseMotion(mapScreenToData(new Point2D.Double(e.getX(), e.getY())) );

    

        getGLContext().gljFree();

    }

    

    /** key pressed handler */

    public void keyPressed(KeyEvent e) {

        if (getGLContext() == null)

            return;

    

        getGLContext().gljMakeCurrent();

    

        checkInputModifiers(e);

        if (display != null) 

            display.canvasInputKeyPress(e.getKeyCode());

    

        getGLContext().gljFree();

    }

     

     

    /** key released handler */ 

    public void keyReleased(KeyEvent e) {

        if (getGLContext() == null)

            return;

        getGLContext().gljMakeCurrent();

        

        checkInputModifiers(e);

        if (display != null) 

            display.canvasInputKeyRelease(e.getKeyCode());

    

        getGLContext().gljFree();

    }

        

    /** key typed handler */    

    public void keyTyped(KeyEvent e) {

    }

    

    /** refreshes canvas */

    public void refresh()

    {

        repaint(new Rectangle(getWidth(), getHeight()));

    }

    

    /** 

     * clears canvas 

     * @param clearColor clearing color

     */

    public void clear(Color clearColor)

    {

        gl.glClearColor((float) clearColor.getRed()/255f, (float) clearColor.getGreen()/255f, (float) clearColor.getBlue()/255f, 0f);

        gl.glClear(GL_COLOR_BUFFER_BIT);

    }

    

    /** 

     * turns on/off quick draw 

     * @param on true to turn on quick draw, false to turn off quick draw 

     */

    public void setQuickDraw(boolean on) {

        quickDraw = on;

    }

    

    /** 

     * sets foreground color of the canvas. 

     * The drawings between this command and next 

     * setForegroundColor command will use this color

     * @param fgColor the color to be set. If this color is translucent, then 

     * the canvas will draw in translucency mode. If it is not, the translucency mode will be turned off

     */

    public void setForegroundColor(Color fgColor) {

        if (fgColor.getAlpha() != 255) {

            // Turn on Translucency

            gl.glEnable(GL_BLEND);

            gl.glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

    

            gl.glColor4d(fgColor.getRed()/255.0, fgColor.getGreen()/255.0, fgColor.getBlue()/255.0, fgColor.getAlpha()/255.0);

        } 

        else {

            // Turn off Translucency

            gl.glDisable(GL_BLEND);

            gl.glColor3d(fgColor.getRed()/255.0, fgColor.getGreen()/255.0, fgColor.getBlue()/255.0);

        }

    }

    

    /** 

     * turns on/off smooth shade model

     * @param on true to turn on smooth shade model, false to turn it off

     */

    public void setSmoothShadeModel(boolean on) {

        if (on)

            gl.glShadeModel(GL_SMOOTH);

        else

            gl.glShadeModel(GL_FLAT);    

    }

    

    /** 

     * sets the width of lines to be drawn 

     * @param w line width to be set 

     */

    public void setLineWidth(double w) {

        gl.glLineWidth((float)w);

    }

    

    /** 

     * draws a line

     * @param pt1 one end point of the line 

     * @param pt2 the other end point of the line 

     */

    public void drawLine(Point2D.Double pt1, Point2D.Double pt2)

    {

        gl.glBegin(GL_LINES);

        gl.glVertex2d(pt1.getX(), pt1.getY());

        gl.glVertex2d(pt2.getX(), pt2.getY());

        gl.glEnd();

    }

    

    /** 

     * draws a polyline

     * @param pts end points of the polyline

     */

    public void drawLines(Point2D.Double[] pts)

    {

        int length = pts.length;

        if (length <= 0)

            return;

        gl.glBegin(GL_LINE_STRIP);

        gl.glVertex2d(pts[0].getX(), pts[0].getY());

        for (int i = 1; i < length; i++) {

            gl.glVertex2d(pts[i].getX(), pts[i].getY());

        }

        gl.glEnd();

    }

    

    /** 

     * sets the size of points to be drawn 

     * @param size point size to be set 

     */

    public void setPointSize(double size) {

        gl.glPointSize((float) size);

    }

    

    /**

     * draws a point

     * @param pt position of the point

     */

    public void drawPoint(Point2D.Double pt)

    {

        gl.glBegin(GL_POINTS);

        gl.glVertex2d(pt.getX(), pt.getY());

        gl.glEnd();

    }

    

    /**

     * draws multiple points

     * @param pts position of the points

     */

    public void drawPoints( Point2D.Double[] pts)

    {

        gl.glBegin(GL_POINTS);

        for (int i=0;i<pts.length;i++) {

            gl.glVertex2d( pts[i].getX(), pts[i].getY());

        }

        gl.glEnd();

    }

    

    /** 

     * draws a polygon

     * @param pts end points of the polygon

     */

    public void drawPolygon(Point2D.Double[] pts)

    {

        int length = pts.length;

        gl.glBegin(GL_LINE_LOOP);

        for (int i = 0; i < length; i++) {

            gl.glVertex2d(pts[i].getX(), pts[i].getY());

        }

        gl.glEnd();

    }

        

    /** 

     * draws a Rectangle

     * @param pt1 one end point of the rectangle

     * @param pt2 the diagonal end point of pt1

     */

    public void drawRectangle(Point2D.Double pt1, Point2D.Double pt2)

    {

        gl.glBegin(GL_LINE_LOOP);

        gl.glVertex2d(pt1.getX(), pt1.getY());

        gl.glVertex2d(pt1.getX(), pt2.getY());

        gl.glVertex2d(pt2.getX(), pt2.getY());

        gl.glVertex2d(pt2.getX(), pt1.getY());

        gl.glVertex2d(pt1.getX(), pt1.getY());

        gl.glEnd();

    }

        

    /** 

     * fills a Rectangle

     * @param pt1 one end point of the rectangle

     * @param pt2 the diagonal end point of pt1

     */

    public void fillRectangle(Point2D.Double pt1, Point2D.Double pt2)

    {

        gl.glBegin(GL_QUADS);

        gl.glVertex2d(pt1.getX(), pt1.getY());

        gl.glVertex2d(pt1.getX(), pt2.getY());

        gl.glVertex2d(pt2.getX(), pt2.getY());

        gl.glVertex2d(pt2.getX(), pt1.getY());

        gl.glEnd();

    }

      

       

    /** 

     * draws a quad strip     

     * @param hiPts higher end points

     * @param loPts lower end points

     */

    public void drawQuadStrip(Point2D.Double[] hiPts, Point2D.Double[] loPts)

    {

        int len = hiPts.length;

        if ( len > loPts.length )

            len = loPts.length;

            

        gl.glBegin(GL_QUAD_STRIP);

        for (int i = 0; i < len; i++)

        {

            gl.glVertex2d(hiPts[i].getX(), hiPts[i].getY());

            gl.glVertex2d(loPts[i].getX(), loPts[i].getY());

        }

        gl.glEnd();

    }

        

        

    /** gets the height of a letter when it is displayed on this canvas 

     * @param ch the letter to be checked

     * @return the height of the letter when it is displayed on this canvas   

     */

    public double getFontHeight(char ch) {

        int fontHeight = font.getFontHeight('K');

        Point2D.Double screenPt1 = mapDataToScreen(new Point2D.Double(0, 0));

        Point2D.Double screenPt2 = mapDataToScreen(new Point2D.Double(0, 1));

        

        return fontHeight / Math.abs(screenPt1.getY() - screenPt2.getY());

    }

    

    /** 

     * gets the width of a letter when it is displayed on this canvas 

     * @param ch the letter to be checked

     * @return the width of the letter when it is displayed on this canvas   

     */

    public double getFontWidth(char ch) {

        int fontWidth = font.getFontWidth('K');

        Point2D.Double screenPt1 = mapDataToScreen(new Point2D.Double(0, 0));

        Point2D.Double screenPt2 = mapDataToScreen(new Point2D.Double(1, 0));

        

        return fontWidth / Math.abs(screenPt1.getX() - screenPt2.getX());

    }

    

    /** 

     * draws a string on the canvas along the horizental direction

     * @param basePt the lower left position of the string on the canvas

     * @param maxWidth the maximum width of the string. 

     * If the total width of the string will exceed this value, only  

     * part of the string will be shown (the ending letters are cut) 

     * to make sure that this string won't 

     * exceed the max width

     * @param text the string to be shown

     */ 

    public void drawHoriString(Point2D.Double basePt, double maxWidth, String text) {        

        Point2D.Double screenBasePt = mapDataToScreen(basePt);

        Point2D.Double screenMaxPt = mapDataToScreen(new Point2D.Double(basePt.getX() + maxWidth, basePt.getY()));

        int screenMaxWidth = (int)Math.abs(screenMaxPt.getX() - screenBasePt.getX());

        

        int fontWidth = font.getFontWidth('W');

        int maxFontNum = screenMaxWidth / fontWidth;

      

        if (maxFontNum == 0)

            return;

        if (maxFontNum < text.length()) {

            font.printHoriString(gl, basePt.getX(), basePt.getY(), text.substring(0, maxFontNum));

        }   

        else {

            font.printHoriString(gl, basePt.getX(), basePt.getY(), text);

        }

    }

    

    /** 

     * draws a string on the canvas along the vertical direction

     * @param basePt the lower left position of the first letter of the string on the canvas

     * @param maxHeight the maximum height of the string. 

     * If the total height of the string will exceed this value, only  

     * part of the string will be shown (the ending letters are cut) 

     * to make sure that this string won't 

     * exceed the max height

     * @param text the string to be shown

     */ 

    public void drawVertString(Point2D.Double basePt, double maxHeight, String text) {        

        Point2D.Double screenBasePt = mapDataToScreen(basePt);

        Point2D.Double screenMaxPt = mapDataToScreen(new Point2D.Double(basePt.getX(), 

                                                     basePt.getY() + maxHeight));

        int screenMaxHeight = (int)Math.abs(screenMaxPt.getY() - screenBasePt.getY());

        

        int fontHeight = font.getFontHeight('K');

        int maxFontNum = screenMaxHeight / fontHeight;

      

        if (maxFontNum == 0)

            return;

        if (maxFontNum < text.length()) {

            font.printVertString(gl, basePt.getX(), basePt.getY(), text.substring(0, maxFontNum));

        }   

        else {

            font.printVertString(gl, basePt.getX(), basePt.getY(), text);

        }

    }

    

    /** maps a point from screen space to data space 

     * @param sc a 2D point in the screen space

     * @return the mapping of the screen space point in data space 

     */

    public Point2D.Double mapScreenToData(Point2D.Double sc) {

        double[] modelMatrix = new double[16];

        double[] projMatrix = new double[16];

        int[] viewport = new int[4];

    

        getGLContext().gljMakeCurrent();

        gl.glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);

        gl.glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);

        gl.glGetIntegerv(GL_VIEWPORT,viewport);

    

        double[] wcX = new double[1];

        double[] wcY = new double[1];

        double[] wcZ = new double[1];

        int ret = glu.gluUnProject(sc.getX(), sc.getY(), 0, modelMatrix, projMatrix, viewport, wcX, wcY, wcZ);

        getGLContext().gljFree();

        return new Point2D.Double(wcX[0],wcY[0]);

    

    }

    

    /** maps a point from data space to screen space 

     * @param p a 2D point in the data space

     * @return the mapping of the data space point in screen space 

     */

    public Point2D.Double mapDataToScreen(Point2D.Double p)

    {

        double[] modelMatrix = new double[16];

        double[] projMatrix= new double[16];

        int[] viewport= new int[4];

        gl.glGetDoublev(GL_MODELVIEW_MATRIX,modelMatrix);

        gl.glGetDoublev(GL_PROJECTION_MATRIX,projMatrix);

        gl.glGetIntegerv(GL_VIEWPORT,viewport);

    

        double[] scX = new double[1];

        double[] scY = new double[1];

        double[] scZ = new double[1];

        int ret = glu.gluProject(p.getX(), p.getY(), 0, modelMatrix,projMatrix, viewport, scX, scY,scZ);

        return new Point2D.Double(scX[0],scY[0]);

    }

    

    

    /**

     * Sets tha canvas cursor as a hand shape

     */

    public void setHandCursor() {

        Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);

        setCursor(handCursor);

    }

    

    /**

     * Sets tha canvas cursor as moving shape

     */

    public void setMoveCursor() {

        Cursor moveCursor = new Cursor(Cursor.MOVE_CURSOR);

        setCursor(moveCursor);

    }

    

    /**

     * Sets tha canvas cursor as default shape

     */

    public void setDefaultCursor() {

        Cursor normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);

        setCursor(normalCursor);

    }

    

    /** Pans the canvas 

     * @param prevPt a point in the screen space

     * @param pt the position of prevPt after the panning operation

     */

    public void pan(Point2D.Double prevPt, Point2D.Double pt)

    {

        setTranslate(translate.getX() + pt.getX() - prevPt.getX(),

                              translate.getY() + pt.getY() - prevPt.getY());              

    }

    

    

    /** Sets translate para of the canva 

     * @param x x value of the translate para

     * @param y y value of the translate para

     */

    public void setTranslate(double x, double y) {

        translate.setLocation(x, y);

        positionHoriScrollBar();

        positionVertScrollBar();

    }

    

    /** Sets zoom para of the canva 

     * @param x x value of the zoom para

     * @param y y value of the zoom para

     */

    public void setZoom(double x, double y) {

        zoom.setLocation(x, y);

        positionHoriScrollBar();

        positionVertScrollBar();

    }

    

    

    /** Sets hori scrollbar according to current translate para and zoom para */

    public void positionHoriScrollBar() {

        if (horiScrollBar == null) 

            return;

        int max, min, value, extent;

        min = 0; 

        max = scrollStepNum - 1;

        value = (int) ((- 1 * translate.getX()) * scrollStepNum);

        extent = (int) (1. / zoom.getX() * 1000);

    

        oldHoriScrollValue = value;

        

        horiScrollBar.setMaximum(max);

        horiScrollBar.setMinimum(min);

        horiScrollBar.setValue(value);

        horiScrollBar.setVisibleAmount(extent);

    }

    

    /** Sets vert scrollbar according to current translate para and zoom para */

    public void positionVertScrollBar() {

        if (vertScrollBar == null) 

            return;

            

        int max, min, value, extent;

        min = 0; 

        max = scrollStepNum - 1;

        extent = (int) (1. / zoom.getY() * 1000);

        value = scrollStepNum - extent + (int) (translate.getY() * scrollStepNum);

    

        oldVertScrollValue = value;

        

        vertScrollBar.setMaximum(max);

        vertScrollBar.setMinimum(min);

        vertScrollBar.setValue(value);

        vertScrollBar.setVisibleAmount(extent);

    }

    

    

    /** hori scrollbar event handler */

    public void horiScrollBarChanged() {

        if (horiScrollBar == null) 

            return;

            

        int newValue = horiScrollBar.getValue();

        

        if (newValue != oldHoriScrollValue) {

            translate.setLocation(-1 * (double)(newValue) / scrollStepNum,  translate.getY());    

            oldHoriScrollValue = newValue;

            refresh();

        }

    }

    

    /** vert scrollbar event handler */

    public void vertScrollBarChanged() {

        if (vertScrollBar == null) 

            return;

            

        int newValue = vertScrollBar.getValue();

        

        if (newValue != oldVertScrollValue) {

            translate.setLocation(translate.getX(), (double)(newValue + vertScrollBar.getVisibleAmount() - scrollStepNum) / scrollStepNum);    

            oldVertScrollValue = newValue;

            refresh();

        }

    }

}

