package xmdv.xmdv_interface;

/***********************************************************
**  CopyRight (c) by Xmdv Group, CS Department of WPI, 2003
**  All Rights Reserved.
**
**  Author: Jing Yang
**  Date:   06/2003
***********************************************************/

import gl4java.GLContext;

import java.awt.FileDialog;
import java.awt.event.*;
import java.io.File;
import java.io.FilenameFilter;

import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.border.BevelBorder;
import javax.swing.border.SoftBevelBorder;

import xmdv.dataproc.Dataset;
import xmdv.graphics.*;

/**
 * This class defines the main frame of xmdv_light_java
 *
 * @author  Jing Yang
 * @version v1.0
 */
public class mainFrame extends javax.swing.JFrame
                       implements FocusListener, ActionListener {
    // Variables declaration
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JScrollBar jScrollBar1;
    private javax.swing.JRadioButton jRadioButton3;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JRadioButton jRadioButton4;
    private javax.swing.JScrollBar jScrollBar2;
    private javax.swing.JRadioButton jRadioButton2;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.ButtonGroup displaymode_buttongroup;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuBar jMenuBar1;
    private grCanvas canvas;
    
    private final String flatParcoordString = "flat parcoord";
    private final String flatGlyphString = "flat glyph";
    private final String flatScatterString = "flat scatter";
    private final String flatDimstackString = "flat dimstack";

    private final String openString = "open";
    private final String closeString = "close";
    
    private Dataset dataset = null;
    // End of variables declaration

    /** Creates new mainFrame */
    public mainFrame() {
        initComponents();
        this.setSize(750, 700);
        
        //set canvas scroll bars
        canvas.setHoriScrollBar(jScrollBar2);
        canvas.setVertScrollBar(jScrollBar1);
        jScrollBar1.addAdjustmentListener(canvas);
        jScrollBar2.addAdjustmentListener(canvas);
        //set canvas textarea
        canvas.setTextArea(jTextArea1);

        //set init dataset and display type
        setDataset("data" + java.io.File.separator + "iris.okc");
        setDisplay(flatParcoordString);
        
        //allow canvas to receive key events
        addFocusListener(this);
        setFocusable(true);
        jTextArea1.setFocusable(false);
        jRadioButton1.setFocusable(false);
        jRadioButton2.setFocusable(false);
        jRadioButton3.setFocusable(false);
        jRadioButton4.setFocusable(false);
    }

    /**
     * Sets the dataset of the canvas
     * @param file name of the dataset with full path
     */
    private void setDataset(String file) {
        if (file != null) {
            dataset = new Dataset();
            if (dataset.load(file)) {
                dataset.addCanvas(canvas);
            }
            else {
                System.out.println("Loading file failed.");
                dataset = null;
            }
        }
        else {
            dataset = null;
        }
         
        if (canvas.getDisplay() != null) {
            canvas.getDisplay().setDataset(dataset);
        }
        canvas.refresh(); 
    }
        

    
    /** This method is called from within the constructor to initialize the form.
     */
    private void initComponents() {
        setTitle("xmdv_light");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        java.awt.event.ActionListener buttonListener = new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonGroupActionPerformed(evt);
            }
        };


        //text area for showing messages. 
        jTextArea1 = new javax.swing.JTextArea();
        jTextArea1.setBackground(new java.awt.Color(204, 204, 204));
        jTextArea1.setEditable(false);
        
        //scrollpane that holds the text area. 
        jScrollPane2 = new javax.swing.JScrollPane();
        jScrollPane2.setBorder(new javax.swing.border.EtchedBorder());
        jScrollPane2.setViewportView(jTextArea1);

        //buttons for selecting display type of canvas
        jRadioButton1 = new javax.swing.JRadioButton();
        jRadioButton2 = new javax.swing.JRadioButton();
        jRadioButton3 = new javax.swing.JRadioButton();
        jRadioButton4 = new javax.swing.JRadioButton();
        displaymode_buttongroup = new javax.swing.ButtonGroup();

        jRadioButton1.setBackground(new java.awt.Color(204, 204, 204));
        jRadioButton1.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jRadioButton1.setBorderPainted(true);
        jRadioButton1.setIcon(new javax.swing.ImageIcon("images" + java.io.File.separator + "imgpa.gif"));
        jRadioButton1.setActionCommand(flatParcoordString);
        jRadioButton1.addActionListener(buttonListener);

        jRadioButton2.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jRadioButton2.setBorderPainted(true);
        jRadioButton2.setIcon(new javax.swing.ImageIcon("images" + java.io.File.separator + "imgst.gif"));
        jRadioButton2.setActionCommand(flatGlyphString);
        jRadioButton2.addActionListener(buttonListener);

        jRadioButton3.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jRadioButton3.setBorderPainted(true);
        jRadioButton3.setIcon(new javax.swing.ImageIcon("images" + java.io.File.separator + "imgsc.gif"));
        jRadioButton3.setActionCommand(flatScatterString);
        jRadioButton3.addActionListener(buttonListener);

        jRadioButton4.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jRadioButton4.setBorderPainted(true);
        jRadioButton4.setIcon(new javax.swing.ImageIcon("images" + java.io.File.separator + "imgdi.gif"));
        jRadioButton4.setActionCommand(flatDimstackString);
        jRadioButton4.addActionListener(buttonListener);

        displaymode_buttongroup.add(jRadioButton1);
        displaymode_buttongroup.add(jRadioButton2);
        displaymode_buttongroup.add(jRadioButton3);
        displaymode_buttongroup.add(jRadioButton4);

        //panel for containing selecting buttons
        jPanel1 = new javax.swing.JPanel();
        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.Y_AXIS));
        jPanel1.add(jRadioButton1);
        jPanel1.add(jRadioButton2);
        jPanel1.add(jRadioButton3);
        jPanel1.add(jRadioButton4);

        //scrollable canvas area
        canvas = new grCanvas();

        jScrollBar1 = new javax.swing.JScrollBar();
        jScrollBar2 = new javax.swing.JScrollBar();

        jScrollBar1.setToolTipText("null");
        jScrollBar2.setOrientation(javax.swing.JScrollBar.HORIZONTAL);

        jPanel2 = new javax.swing.JPanel();
        jPanel2.setLayout(new java.awt.BorderLayout());
        jPanel2.add(jScrollBar1, java.awt.BorderLayout.EAST);
        jPanel2.add(jScrollBar2, java.awt.BorderLayout.SOUTH);
        jPanel2.add(canvas, java.awt.BorderLayout.CENTER);

        //menus
        JMenuItem menuItem;

        jMenuBar1 = new javax.swing.JMenuBar();

        jMenu1 = new javax.swing.JMenu();
        jMenu1.setText("File");
        jMenuBar1.add(jMenu1);

        menuItem = new JMenuItem("Open");
        menuItem.addActionListener(this);
        menuItem.setActionCommand(openString);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_1, ActionEvent.ALT_MASK));
        jMenu1.add(menuItem);

        menuItem = new JMenuItem("Close");
        menuItem.addActionListener(this);
        menuItem.setActionCommand(closeString);
        jMenu1.add(menuItem);

        getContentPane().add(jScrollPane2, java.awt.BorderLayout.SOUTH);
        getContentPane().add(jPanel1, java.awt.BorderLayout.EAST);
        getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);

        setJMenuBar(jMenuBar1);

        pack();
    }
    
    /**
     * Handles menu events.  
     * @param evt menu event
     */
    public void actionPerformed(ActionEvent evt) {
        String str = evt.getActionCommand();
        //open menu
        if (str.equals(openString)) {
            FileDialog dialog = new FileDialog(this, "Load an okc file");
            dialog.setFilenameFilter(new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    return name.endsWith(".okc");
                }
            });
            dialog.show();
            if (dialog.getFile() == null)
                return;
            setDataset(dialog.getDirectory() + File.separator + dialog.getFile());
        } 
        //close menu
        else if (str.equals(closeString)) {
            setDataset(null);
        }        
    }

    /**
     * Sets the display type of canvas.  
     * @param str name of a display type 
     */
    private void setDisplay(String str) {
        //sets the status of the display selection buttons
        jRadioButton1.setBorder(new SoftBevelBorder(str.equals(flatParcoordString) ? BevelBorder.LOWERED : BevelBorder.RAISED));
        jRadioButton2.setBorder(new SoftBevelBorder(str.equals(flatGlyphString) ? BevelBorder.LOWERED : BevelBorder.RAISED));
        jRadioButton3.setBorder(new SoftBevelBorder(str.equals(flatScatterString) ? BevelBorder.LOWERED : BevelBorder.RAISED));
        jRadioButton4.setBorder(new SoftBevelBorder(str.equals(flatDimstackString) ? BevelBorder.LOWERED : BevelBorder.RAISED));

        FlatDisplay dis = null;
        if (str.equals(flatParcoordString)) {
            dis = new FlatParcoord(canvas, new ColorManager(), dataset);
        } else if (str.equals(flatGlyphString)) {
            dis = new FlatGlyph(canvas, new ColorManager(), dataset);
        } else if (str.equals(flatScatterString)) {
            dis = new FlatScatter(canvas, new ColorManager(), dataset);
        } else if (str.equals(flatDimstackString)) {
            dis = new FlatDimstack(canvas, new ColorManager(), dataset);
        }
        canvas.setDisplay(dis);
        canvas.refresh();
    }        
    
    private void buttonGroupActionPerformed(java.awt.event.ActionEvent evt) {
        String str = evt.getActionCommand();
        setDisplay(str);        
    }

    private void jRadioButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButton1ActionPerformed
        // Add your handling code here:
    }
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }

    public void focusGained(FocusEvent e) 
    {
        canvas.requestFocus();
    }
    
    public void focusLost(FocusEvent e) 
    {
    }

    
     /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        if (! GLContext.doLoadNativeLibraries(null, null, null)) {
            System.out.println("could not load native libs");
            return;
        }
        new mainFrame().show();
    }
}
